"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easing = _interopRequireDefault(require("./easing"));

var _cancelEvents = _interopRequireDefault(require("./cancel-events"));

var _scrollEvents = _interopRequireDefault(require("./scroll-events"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var scrollEasing = _easing.default.scroll;
var __currentPositionY = 0;
var __startPositionY = 0;
var __targetPositionY = 0;
var __progress = 0;
var __duration = 0;
var __cancel = false;

var __target;

var __to;

var __start;

var __deltaTop;

var __percent;

var __delayTimeout;
/*
 * Function helper
 */


var functionWrapper = function functionWrapper(value) {
  return typeof value === 'function' ? value : function () {
    return value;
  };
};
/*
 * Sets the cancel trigger
 */


_cancelEvents.default.register(function () {
  __cancel = true;
});
/*
 * Wraps window properties to allow server side rendering
 */


var currentWindowProperties = function currentWindowProperties() {
  if (typeof window !== 'undefined') {
    return window.requestAnimationFrame || window.webkitRequestAnimationFrame;
  }
};
/*
 * Helper function to never extend 60fps on the webpage.
 */


var requestAnimationFrameHelper = function () {
  return currentWindowProperties() || function (callback, element, delay) {
    window.setTimeout(callback, delay || 1000 / 60, new Date().getTime());
  };
}();

var currentPositionY = function currentPositionY() {
  var supportPageOffset = window.pageXOffset !== undefined;
  var isCSS1Compat = (document.compatMode || '') === 'CSS1Compat';
  return supportPageOffset ? window.pageYOffset : isCSS1Compat ? document.documentElement.scrollTop : document.body.scrollTop;
};

var pageHeight = function pageHeight() {
  var body = document.body;
  var html = document.documentElement;
  return Math.max(body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight);
};

var animateTopScroll = function animateTopScroll(timestamp) {
  // Cancel on specific events
  if (__cancel) {
    return;
  }

  __deltaTop = Math.round(__targetPositionY - __startPositionY);

  if (__start === null) {
    __start = timestamp;
  }

  __progress = timestamp - __start;
  __percent = __progress >= __duration ? 1 : scrollEasing(__progress / __duration);
  __currentPositionY = __startPositionY + Math.ceil(__deltaTop * __percent);
  window.scrollTo(0, __currentPositionY);

  if (__percent < 1) {
    requestAnimationFrameHelper.call(window, animateTopScroll);
    return;
  }

  if (_scrollEvents.default.registered.end) {
    _scrollEvents.default.registered.end(__to, __target, __currentPositionY);
  }
};

var startAnimateTopScroll = function startAnimateTopScroll(y, options, to, target) {
  window.clearTimeout(__delayTimeout);
  __start = null;
  __cancel = false;
  __startPositionY = currentPositionY();
  __targetPositionY = options.absolute ? y : y + __startPositionY;
  __deltaTop = Math.round(__targetPositionY - __startPositionY);
  __duration = functionWrapper(options.duration)(__deltaTop);
  __duration = isNaN(parseFloat(__duration)) ? 1000 : parseFloat(__duration);
  __to = to;
  __target = target;

  if (options && options.delay > 0) {
    __delayTimeout = window.setTimeout(function animate() {
      requestAnimationFrameHelper.call(window, animateTopScroll);
    }, options.delay);
    return;
  }

  requestAnimationFrameHelper.call(window, animateTopScroll);
};

var scrollToTop = function scrollToTop(options) {
  startAnimateTopScroll(0, Object.assign(options || {}, {
    absolute: true
  }));
};

var scrollTo = function scrollTo(toY, options) {
  startAnimateTopScroll(toY, Object.assign(options || {}, {
    absolute: true
  }));
};

var scrollToBottom = function scrollToBottom(options) {
  startAnimateTopScroll(pageHeight(), Object.assign(options || {}, {
    absolute: true
  }));
};

var scrollMore = function scrollMore(toY, options) {
  startAnimateTopScroll(currentPositionY() + toY, Object.assign(options || {}, {
    absolute: true
  }));
};

var _default = {
  animateTopScroll: startAnimateTopScroll,
  scrollToTop: scrollToTop,
  scrollToBottom: scrollToBottom,
  scrollTo: scrollTo,
  scrollMore: scrollMore
};
exports.default = _default;
module.exports = exports["default"];